#include "scenetree.h"
#include "resourcemanager.h"
#include <pipeline.h>

// Scene

SceneTree::SceneTree() :
    Scene(),
    m_skybox(NULL)
{
    m_simplepipeline = new SimplePipeline();
    m_simplepipeline->setClearColor(glm::vec3(0.1f, 0.4f, 0.25f));
}

SceneTree::~SceneTree()
{

}

void SceneTree::setMainCamera(CameraNode *camera)
{
    m_simplepipeline->setCamera(camera);
}

SceneIterator<Light*>* SceneTree::getLights()
{
    return new ArrayIterator<Light*>(m_lights);
}

SceneIterator<GeometryNode*>* SceneTree::getGeometry()
{
    return new ArrayIterator<GeometryNode*>(m_geometries);
}

void SceneTree::update()
{
    m_root.update();
}

void SceneTree::addObject(ContainerNode *parent, SceneNode *node)
{
    parent->addChild(node);
    Light *light = node->getLight();
    GeometryNode *geometrynode = node->getGeometryNode();
    //TODO : Check for doublon in m_lights et m_geometries
    if (light) m_lights.push_back(light);
    if (geometrynode) m_geometries.push_back(geometrynode);
}

void SceneTree::removeObject(ContainerNode* parent,SceneNode *node){
    parent->removeChild(node);
    Light *l = node->getLight();
    GeometryNode *g = node->getGeometryNode();
    //TODO : remove light and geometrynode from m_lights et m_geometries
    // debug following line
//    m_lights.erase(std::remove(m_lights.begin(),m_lights.end(),&(*l)),m_lights.end());
//    m_geometries.erase(std::remove(m_geometries.begin(),m_geometries.end(),g),m_geometries.end());
}


// Container Node

void ContainerNode::update()
{
    for(SceneNode* sn : m_children)
        sn->update();
}

void ContainerNode::addChild(SceneNode* node)
{
    if(node != NULL)
    {
        m_children.push_back(node);
        node->m_parent = this;
    }
}

void ContainerNode::removeChild(SceneNode* node)
{
    if(node != NULL)
    {
        for(auto it = m_children.begin(); it != m_children.end(); ++it)
        {
            if(*it == node)
            {
                m_children.erase(it);
                break;
            }
        }
    }
}

